# JavaScript/WebGL Scene Format

The top-level defines the following variables (declared with 'var' or
by assigning to an identifier):

  - group : optional map, default = {}
  - SceneScaleV : optional number, default = 1
  - SceneScaleC : optional number, default = 1
  - SceneShiftI : optional number, default = 0
  - scene : optional array, default = []
  - worldPos : optional array of map
  - anim : optional boolean, default = true

The 'scene' and 'group' variable can be either assigned, or modified
(e.g. by push).

Additional definitions may exist for:

  - abbreviating numbers: variables 'a',..,'z', 'A', .., 'Z', and more
    are generated by libcsg to store values used multiple times

  - user definitions for functions used for different groups for
    transformations.

## 'group'

This defines named subscenes.  The key for each group is typically an
integer.  Each value of the 'group' variable map defines a group and
has the following slots:

  - 'tag' : mandatory string;
    Short name of the group, referenced in scenes.

  - 'name' : mandatory string;
    Display name of the group.

  - 'optional' : mandatory boolean;
    Whether this group may be switched off.

  - 'default_off' : mandatory boolean;
    Whether this group should be switched off in the view by default.

  - 'fixed' : mandatory boolean;
    Whether this should be fixed in view, i.e., not be transformed
    by the camera view.

  - 'prio' : mandatory number;
    Priority setting for sorting switching things on/off.

  - 'move' : optional map;
    Definition of dynamic movement (translation) of this group.

  - 'scale' : optional map;
    Definition of dynamic scaling of this group.

  - 'rot' : optional map;
    Definition of dynamic rotation of this group.

For convenience, each value in the 'group' variable should be assigned
to a variable calles 'group_' + the group's tag.  This allows
references from one group to the next inside the group's slots.

### 'move'

  - 'unit' : mandatory array of 3 numbers;
  - 'name' : mandatory string;
  - 'amount' : mandatory number;
  - 'get' : optional function;
  - 'xlat' : optoinal function;

If 'get' is defined, a translation will be done.

### 'scale'

  - 'center' : mandatory array of 3 numbers;
  - 'unit' : mandatory array of 3 numbers;
  - 'get' : optional function;

If 'get' is defined, a scalation will be done.

### 'rot'

  - 'center' : mandatory array of 3 numbers;
  - 'unit' : mandatory array of 3 numbers;
  - 'get' : optional function;

If 'get' is defined, a rotation will be done.

## 'worldPos'

If non-empty, the first entry will be the default world view. All
entries get a button to switch.

An array of matrices defining a standard camera view.  Each entry is a
map with the following slots:

  - 'name' : string;
     The name of the view (for generating a button to switch to this view).
     If this is boolean false, this will not show up as a button.

  - 'mat' : array[16] of float;
     A 16 entry (4x4) matrix with indices 0..3 forming the first column,
     defining the transformation for this view.
     If this is missing, the worldPos matrix will not be reset.

  - 'cam' : array[3] of float;
     A 3 entry matrix with the camera position (in meter [m]).
     If this is missing, the camera position will not be reset.

  - 'view' : array[16] of float;
     A 16 entry (4x4) matrix with indices 0..3 forming the first column,
     defining the camera view.
     If this is missing, the worldView matrix will not be reset.


The default view is the unit matrix.

## 'scene'

Scene is an array of parts of the scene.  Each entry is a map that has
the following slots:

  - 'scaleV' : optional number, local override of `SceneScaleV`

  - 'scaleC' : optional number, local override of `SceneScaleC`

  - 'shiftI' : optional number, local override of `SceneShiftI`

  - 'group' : mandatory map;
     Which groups this part of the scene belongs to.

  - 'vertex' : mandatory array of 3 numbers;
     XYZ vertex coordinates used in this part of the scene.

  - 'normal' : mandatory array of 3 numbers;
     XYZ normal coordinates used in this part of the scene for each vertex.

  - 'color' : mandatory array of 4 numbers;
     RGBA colours used in this part of the scene for each vertex.

  - 'index' : mandatory array of 3 numbers;
     Triangles of this part of the scene, each as 3 indices into the
     'vertex', 'normal', and 'color' arrays.

  - 'xform' : optional array[16] of float:
     A 16 entry (4x4) matrix with indices 0..3 forming the first
     column, to transform the complete scene.  This transformation is
     unconditionally applied to all the coordinates of the scene.  It
     is equivalent to applying the transformation before writing the
     output file.

Each 3 value entry in the 'vertex' and 'normal' arrays and each 4
value entry 'color' array correspond to each other, i.e., the 'normal'
array has to have the same size as the 'vertex' array, and for every 3
values in the 'vertex' array, there must be four values in the 'color'
array.

## 'anim'

Whether to start with activated animation.  Can be set to 'false' to
switch the default off.

### 'group'

A map of key that indexes the global 'group' map and maps to boolean 'true'.
This defines which groups this part of the scene belongs to.

### 'vertex'

A flat array with 3 entries for each vertex of the scene.  The 3
entries are the x, y, and z coordinates of the vertex.  Its actual
coordinate value is the number in the array divided by the global
'SceneScaleV' value.  The unit of the actual value is meters [m].

The maximum length of the 'vertex' array is 3*65535.

### 'normal'

A flat array with 3 entries for each scene vertex's normal.  The 3
entries are the x, y, and z components of the direction vector.  Its
actual value is the number in the array divided by the global
'SceneScaleV' value.  Each normal's actual value should have length 1.

The maximum length of the 'normal' array is 3*65535.

### 'color'

A flat array with 4 entries for each scene vertex's colour.  The 4
entries are the r, g, b, and a values of the colour.  The actual value
used is the number in the array divided by the global 'SceneScaleC'.
The actual value should be between 0 and 1.  For r,g,b, 0 is the
darkest and 1 the brightest.  For a, 1 is full opacity and 0 is full
transparency.

The maximum length of the 'color' array is 4*65535.

### 'index'

A flat array with 3 entries for each triangle of the schene, each
index defining which entry in the vertex, normal, and color arrays to
use for the triangle's three vertices.  The index is 0 based and
counts multi-value entries in the corresponding arrays: vertex and
normals use 3 values for each index, while the color array uses 4
values, i.e., index value 1 refers to the array entries at 0-based
positions 3,4,5 from vertex and normal, and 4,5,6,7 from color.

Each index is stored relative to the last one.  The actual index value
used for a value k stored in the 'index' array is ((k + last) -
SceneShiftI).  This value is used as the 'last' value for the next
index.  The 'last' value for the first entry is 0.

The maximum index value derived from any value of the 'index' array
is 65535.  This is a restriction of the WebGL framework, which only
supports uint16 indices.

## Examples

### Fully Defined Structures

The following represents a centred cuboid with edge length 8 cm in x
direction, 6 cm in y direction and 4 cm in z direction in bright
orange colour at full opacity.  Groups are not used in this example.

```
var group = {};
var sceneScaleV = 10000;
var sceneScaleC = 255;
var sceneShiftI = 4;
var scene_0 = {
   'group':{},
   'vertex':[
       +400, +300, +200,   +400, +300, +200,   +400, +300, +200,
       -400, +300, +200,   -400, +300, +200,   -400, +300, +200,
       +400, -300, +200,   +400, -300, +200,   +400, -300, +200,
       -400, -300, +200,   -400, -300, +200,   -400, -300, +200,
       +400, +300, -200,   +400, +300, -200,   +400, +300, -200,
       -400, +300, -200,   -400, +300, -200,   -400, +300, -200,
       +400, -300, -200,   +400, -300, -200,   +400, -300, -200,
       -400, -300, -200,   -400, -300, -200,   -400, -300, -200,
   ],
   'normal':[
       +10000, 0, 0,       0, +10000, 0,       0, 0, +10000,
       -10000, 0, 0,       0, +10000, 0,       0, 0, +10000,
       +10000, 0, 0,       0, -10000, 0,       0, 0, +10000,
       -10000, 0, 0,       0, -10000, 0,       0, 0, +10000,
       +10000, 0, 0,       0, +10000, 0,       0, 0, -10000,
       -10000, 0, 0,       0, +10000, 0,       0, 0, -10000,
       +10000, 0, 0,       0, -10000, 0,       0, 0, -10000,
       -10000, 0, 0,       0, -10000, 0,       0, 0, -10000,
   ],
   'color':[
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
   ],
   'index':[
       4,10,10, 10,-2,-2,
       1,10,10, 10,-2,-2,
       -4,7,13, 7,1,-5,
       7,7,13,  7,1,-5,
       -4,7,7,  7,1,1,
       13,7,7,  7,1,1
   ]
};
var scene = [
    scene_0
];
```

### Using Push

```
scene.push({
   'group':{},
   `scaleV`:10000,
   `scaleC`:255,
   `shiftI`:4,
   'vertex':[
       +400, +300, +200,   +400, +300, +200,   +400, +300, +200,
       -400, +300, +200,   -400, +300, +200,   -400, +300, +200,
       +400, -300, +200,   +400, -300, +200,   +400, -300, +200,
       -400, -300, +200,   -400, -300, +200,   -400, -300, +200,
       +400, +300, -200,   +400, +300, -200,   +400, +300, -200,
       -400, +300, -200,   -400, +300, -200,   -400, +300, -200,
       +400, -300, -200,   +400, -300, -200,   +400, -300, -200,
       -400, -300, -200,   -400, -300, -200,   -400, -300, -200,
   ],
   'normal':[
       +10000, 0, 0,       0, +10000, 0,       0, 0, +10000,
       -10000, 0, 0,       0, +10000, 0,       0, 0, +10000,
       +10000, 0, 0,       0, -10000, 0,       0, 0, +10000,
       -10000, 0, 0,       0, -10000, 0,       0, 0, +10000,
       +10000, 0, 0,       0, +10000, 0,       0, 0, -10000,
       -10000, 0, 0,       0, +10000, 0,       0, 0, -10000,
       +10000, 0, 0,       0, -10000, 0,       0, 0, -10000,
       -10000, 0, 0,       0, -10000, 0,       0, 0, -10000,
   ],
   'color':[
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
       255,128,0,255,      255,128,0,255,      255,128,0,255,
   ],
   'index':[
       4,10,10, 10,-2,-2,
       1,10,10, 10,-2,-2,
       -4,7,13, 7,1,-5,
       7,7,13,  7,1,-5,
       -4,7,7,  7,1,1,
       13,7,7,  7,1,1
   ]
});
```
